
/*******************************************************************************
*
* File check3.c
*
* Copyright (C) 2015 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the programs cprop() and random_csrc().
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "mpi.h"
#include "sl3.h"
#include "random.h"
#include "pmdb.h"
#include "utils.h"
#include "lattice.h"
#include "linalg.h"
#include "cflds.h"
#include "global.h"

#define N0 (NPROC0*L0)
#define N1 (NPROC1*L1)
#define N2 (NPROC2*L2)
#define N3 (NPROC3*L3)


static void new_fld(sl3_alg_dble *cd)
{
   gauss_dble((double*)(cd),16*NSPIN);
}


static int is_equal(sl3_alg_dble *cd1,sl3_alg_dble *cd2)
{
   int it;

   it=1;

   it&=((*cd1).c1.re==(*cd2).c1.re);
   it&=((*cd1).c1.im==(*cd2).c1.im);
   it&=((*cd1).c2.re==(*cd2).c2.re);
   it&=((*cd1).c2.im==(*cd2).c2.im);
   it&=((*cd1).c3.re==(*cd2).c3.re);
   it&=((*cd1).c3.im==(*cd2).c3.im);
   it&=((*cd1).c4.re==(*cd2).c4.re);
   it&=((*cd1).c4.im==(*cd2).c4.im);
   it&=((*cd1).c5.re==(*cd2).c5.re);
   it&=((*cd1).c5.im==(*cd2).c5.im);
   it&=((*cd1).c6.re==(*cd2).c6.re);
   it&=((*cd1).c6.im==(*cd2).c6.im);
   it&=((*cd1).c7.re==(*cd2).c7.re);
   it&=((*cd1).c7.im==(*cd2).c7.im);
   it&=((*cd1).c8.re==(*cd2).c8.re);
   it&=((*cd1).c8.im==(*cd2).c8.im);

   return it;
}


static int cmp_flds1(sl3_alg_dble *cd1,sl3_alg_dble *cd2)
{
   int ix,it;

   it=1;

   for (ix=0;ix<VOLUME;ix++)
   {
      it&=is_equal(cd1,cd2);
      cd1+=1;
      cd2+=1;
   }

   return it;
}


static int chk_bnd(sl3_alg_dble *cd)
{
   int bc,ix,t,it;
   int cp[4],root;
   sl3_alg_dble ct0;

   bc=bc_type();
   it=1;

   if (bc==1)
   {
      cp[0]=0;
      cp[1]=0;
      cp[2]=0;
      cp[3]=0;
      root=ipr_global(cp);

      ix=ipt[0];
      ct0=cd[ix];
      MPI_Bcast(&ct0,16,MPI_DOUBLE,root,MPI_COMM_WORLD);

      for (ix=0;ix<VOLUME;ix++)
      {
         t=global_time(ix);

         if (t==0)
            it&=is_equal(cd+ix,&ct0);
      }
   }

   return it;
}


static double dev_sl3(sl3_alg_dble *X,sl3_alg_dble *Y)
{
   double d,dmax;

   dmax=0.0;

   d=fabs((*X).c1.re-(*Y).c1.re)+fabs((*X).c1.im-(*Y).c1.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c2.re-(*Y).c2.re)+fabs((*X).c2.im-(*Y).c2.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c3.re-(*Y).c3.re)+fabs((*X).c3.im-(*Y).c3.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c4.re-(*Y).c4.re)+fabs((*X).c4.im-(*Y).c4.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c5.re-(*Y).c5.re)+fabs((*X).c5.im-(*Y).c5.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c6.re-(*Y).c6.re)+fabs((*X).c6.im-(*Y).c6.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c7.re-(*Y).c7.re)+fabs((*X).c7.im-(*Y).c7.im);
   if (d>dmax)
      dmax=d;
   d=fabs((*X).c8.re-(*Y).c8.re)+fabs((*X).c8.im-(*Y).c8.im);
   if (d>dmax)
      dmax=d;

   return dmax;
}


static double cmp_flds2(sl3_alg_dble *cd1,sl3_alg_dble *cd2)
{
   int bc,ix,t;
   double d,dmax,r,rt[N0];
   sl3_alg_dble ct0,X;

   bc=bc_type();
   dmax=0.0;

   if (bc==1)
   {
      for (t=0;t<N0;t++)
         rt[t]=1.0-(double)(t)/(double)(N0);

      _sl3_alg_mulr_assign(X,0.0,cd2[0]);

      for (ix=0;ix<VOLUME;ix++)
      {
         t=global_time(ix);
         _sl3_alg_mulr_add_assign(X,rt[t],cd2[ix]);
      }

      if (NPROC>1)
      {
         MPI_Reduce(&X,&ct0,16,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD);
         MPI_Bcast(&ct0,16,MPI_DOUBLE,0,MPI_COMM_WORLD);
      }
      else
         ct0=X;

      r=1.0/((double)(N0*N1)*(double)(N2*N3));
      _sl3_alg_mulr_assign(ct0,r,ct0);

      for (ix=0;ix<VOLUME;ix++)
      {
         t=global_time(ix);

         if (t==0)
            d=dev_sl3(&ct0,cd1+ix);
         else
         {
            _sl3_alg_mulr_assign(X,rt[t],ct0);
            _sl3_alg_add_assign(X,cd2[ix]);
            d=dev_sl3(&X,cd1+ix);
         }

         if (d>dmax)
            dmax=d;
      }
   }
   else
   {
      for (ix=0;ix<VOLUME;ix++)
      {
         t=global_time(ix);

         if (t==0)
         {
            _sl3_alg_mulr_assign(X,0.5,cd1[ix]);
            d=dev_sl3(&X,cd2+ix);
         }
         else
            d=dev_sl3(cd1+ix,cd2+ix);

         if (d>dmax)
            dmax=d;
      }
   }

   if (NPROC>1)
   {
      d=dmax;
      MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
      MPI_Bcast(&dmax,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   }

   return dmax;
}


static double cmp_flds3(sl3_alg_dble *cd1,sl3_alg_dble *cd2)
{
   int ix;
   double d,dmax;

   dmax=0.0;

   for (ix=0;ix<VOLUME;ix++)
   {
      d=dev_sl3(cd1+ix,cd2+ix);
      if (d>dmax)
         dmax=d;
   }

   if (NPROC>1)
   {
      d=dmax;
      MPI_Reduce(&d,&dmax,1,MPI_DOUBLE,MPI_MAX,0,MPI_COMM_WORLD);
      MPI_Bcast(&dmax,1,MPI_DOUBLE,0,MPI_COMM_WORLD);
   }

   return dmax;
}


int main(int argc,char *argv[])
{
   int my_rank,bc,ie;
   double dev;
   sl3_alg_dble **wcd;
   FILE *flog=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      flog=freopen("check3.log","w",stdout);

      printf("\n");
      printf("Check of the programs cprop() and random_csrc()\n");
      printf("-----------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      bc=find_opt(argc,argv,"-bc");

      if (bc!=0)
         error_root(sscanf(argv[bc+1],"%d",&bc)!=1,1,"main [check3.c]",
                    "Syntax: check3 [-bc <type>]");
      fflush(flog);
   }

   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   set_th_parms(LYM,bc,0);
   print_th_parms();
   start_ranlux(0,12345);
   geometry();
   alloc_wcd(4);

   wcd=reserve_wcd(4);
   new_fld(wcd[0]);
   new_fld(wcd[1]);
   new_fld(wcd[2]);
   new_fld(wcd[3]);

   set_cdbc(wcd[0]);
   assign_calg2calg(VOLUME,wcd[0],wcd[3]);
   cprop(wcd[0],wcd[1]);
   ie=cmp_flds1(wcd[0],wcd[3]);
   error(ie!=1,1,"main [check3.c]",
         "Input field is not unchanged");

   ie=chk_bnd(wcd[1]);
   error(ie!=1,1,"main [check3.c]",
         "Output of cprop does not satisfy the boundary conditions");

   cprop(wcd[0],wcd[0]);
   ie=cmp_flds1(wcd[0],wcd[1]);
   error(ie!=1,1,"main [check3.c]",
         "Incorrect in place application of cprop()");

   dfp(wcd[1],wcd[2]);
   dev=cmp_flds2(wcd[2],wcd[3]);

   if (my_rank==0)
      printf("Check of cprop: comparison with D_FP = %.1e\n",dev);

   new_fld(wcd[0]);
   assign_calg2calg(VOLUME,wcd[0],wcd[1]);
   set_cdbc(wcd[0]);
   cprop(wcd[0],wcd[2]);
   cprop(wcd[1],wcd[3]);
   dev=cmp_flds3(wcd[2],wcd[3]);

   if (my_rank==0)
      printf("Check of cprop: sensitivity to the boundary values = %.1e\n",dev);

   random_csrc(wcd[1],wcd[0]);
   assign_calgdag2calg(VOLUME,wcd[1],wcd[1]);
   cprop(wcd[1],wcd[1]);
   mulr_add_calg(VOLUME,wcd[1],wcd[0],1.0);
   dev=norm_square_calg(VOLUME,1,wcd[1])/norm_square_calg(VOLUME,1,wcd[0]);

   if (my_rank==0)
      printf("Check of random_csrc: relative deviation   = %.1e\n\n",dev);

   if (my_rank==0)
      fclose(flog);

   MPI_Finalize();
   exit(0);
}
