
/*******************************************************************************
*
* File check1.c
*
* Copyright (C) 2015 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Size of the workspace required by the program valrt().
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <float.h>
#include "mpi.h"
#include "sl3.h"
#include "random.h"
#include "pmdb.h"
#include "lattice.h"
#include "trees.h"
#include "aflds.h"
#include "ispt.h"
#include "global.h"

static int my_rank,bc,Nf,order,iroot;
static int nw[4],nwmx[4];
static int opv[(int)(PM_TYPES)];
static double *pv[(int)(PM_TYPES)];
static theory_t th;
static char theory[NAME_SIZE];
static FILE *fin=NULL,*flog=NULL;


static void set_th(void)
{
   if (strcmp("PHI4",theory)==0)
      th=PHI4;
   else if (strcmp("YM",theory)==0)
      th=YM;
   else if (strcmp("QCD",theory)==0)
      th=QCD;
   else if (strcmp("LYM",theory)==0)
      th=LYM;
   else if (strcmp("LQCD",theory)==0)
      th=LQCD;
   else
      error(1,1,"set_th [check1.c]","Unknown theory");
}


static void read_theory_parms(void)
{
   find_section("Theory");
   read_line("theory","%s",theory);
   read_line("bc","%d",&bc);
   read_line("Nf","%d",&Nf);

   find_section("ISPT");
   read_line("order","%d",&order);
   read_line("iroot","%d",&iroot);
   set_th();
}


static void read_infile(void)
{
   int ith;

   if (my_rank==0)
   {
      fin=freopen("check1.in","r",stdin);
      error_root(fin==NULL,1,"read_infile [check1.c]",
                 "Unable to open input file");
      read_theory_parms();
      fclose(fin);
   }

   ith=(int)(th);
   MPI_Bcast(&ith,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&Nf,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&order,1,MPI_INT,0,MPI_COMM_WORLD);
   MPI_Bcast(&iroot,1,MPI_INT,0,MPI_COMM_WORLD);
   th=(theory_t)(ith);
}


static void default_pv(void)
{
   int i,pmax;
   double *p;

   pmax=(int)(PM_TYPES);
   p=malloc(pmax*(order+1)*sizeof(*p));

   for (i=0;i<(pmax*(order+1));i++)
      p[i]=0.0;

   for (i=0;i<pmax;i++)
   {
      opv[i]=order;
      pv[i]=p;
      p+=order+1;
   }

   pv[(int)(C0)][0]=1.0;
   pv[(int)(LAMBDA)][0]=1.0;
   pv[(int)(MASS)][0]=0.0;
   pv[(int)(CG)][0]=1.0;
   pv[(int)(CGP)][0]=1.0;
   pv[(int)(CSW)][0]=1.0;
   pv[(int)(CF)][0]=1.0;
   pv[(int)(CFP)][0]=1.0;
}


int main(int argc,char *argv[])
{
   int na0,na1,nlp,io,ilp;
   int n,i,j;
   ltree_t **lta;
   rtree_t *rt;
   tlist_t *tl;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);
   set_error_file("check1.log",0);

   if (my_rank==0)
   {
      flog=freopen("check1.log","w",stdout);

      printf("\n");
      printf("Size of the workspace required by the program valrt()\n");
      printf("-----------------------------------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);
      fflush(flog);
   }

   read_infile();
   set_th_parms(th,bc,Nf);
   default_pv();
   set_ispt_parms(order,iroot,opv,pv);
   print_th_parms();
   print_ispt_parms();
   start_ranlux(0,12345);
   geometry();

   na0=trees(th,order);
   na1=ltrees(th,order);
   set_root(th,iroot);

   if (my_rank==0)
   {
      printf("Total number of basic tree diagrams =   %d\n",na0);
      printf("Total number of labeled tree diagrams = %d\n\n",na1);

      printf("Numbers of required fields:\n\n");
      printf("  order  nloop   SCALAR  GLUON  GHOST  QUARK\n");
   }

   for (io=1;io<=order;io++)
   {
      nlp=nlp_max(th,io);

      for (ilp=0;ilp<=nlp;ilp++)
      {
         tl=select_ltrees(th,io,ilp);

         n=(*tl).n;
         lta=(*tl).lta;

         for (i=0;i<4;i++)
            nwmx[i]=0;

         for (i=0;i<n;i++)
         {
            rt=get_rtree(th,order,lta[i]);
            valrt_wspace(rt,nw);

            for (j=0;j<4;j++)
            {
               if (nw[j]>nwmx[j])
                  nwmx[j]=nw[j];
            }
         }

         if (my_rank==0)
         {
            printf("    %d      %d       ",io,ilp);

            for (i=0;i<4;i++)
               printf("%2d     ",nwmx[i]);

            printf("\n");
         }
      }
   }

   if (my_rank==0)
   {
      printf("\n");
      fclose(flog);
   }

   MPI_Finalize();
   exit(0);
}
