
/*******************************************************************************
*
* File check3.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Contraction of source fields.
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include "utils.h"
#include "permutations.h"
#include "contract.h"

static int k,n,ml,mj,cidl,cidj;
static int *ki,*li,*ji;
static int *nl,*nj,(*rl)[2],(*rj)[2];
static int nvw,nlw,*rkw;
static long nck,ncl,ncj;
static long nmk,nml,nmj;
static line_t *lnl;
static diagram_t *dmw;
static vlist_t vls;
static ltree_t ***lta,**ltd;
static dlist_t ***dls;
static tlist_t ***tls;
static src_legs_t sl;
static vsets_t vs;


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check3.c]","Unknown theory");

   return THEORIES;
}


static void set_dls(theory_t th,int order)
{
   int i,j;
   int n1,n2,nel;
   dlist_t ***dl1,**dl2;

   n1=order+1;
   n2=0;

   for (i=1;i<=order;i++)
      n2+=(nel_max(th,i)+1);

   dl1=malloc(n1*sizeof(*dl1));
   dl2=malloc(n2*sizeof(*dl2));
   error((dl1==NULL)||(dl2==NULL),1,"set_dls [check3.c]",
         "Unable to allocate dlist array");
   dl1[0]=NULL;

   for (i=1;i<=order;i++)
   {
      dl1[i]=dl2;
      nel=nel_max(th,i);

      for (j=0;j<=nel;j++)
      {
         if (j>=2)
            dl2[j]=select_cdgms(th,i,j);
         else
            dl2[j]=NULL;
      }

      dl2+=(nel+1);
   }

   dls=dl1;
}


static void free_dls(void)
{
   free(dls[1]);
   free(dls);
}


static void set_tls(theory_t th,int order)
{
   int i,j;
   int n1,n2,nlp;
   tlist_t ***tl1,**tl2;

   n1=order+1;
   n2=0;

   for (i=1;i<=order;i++)
      n2+=(nlp_max(th,i)+1);

   tl1=malloc(n1*sizeof(*tl1));
   tl2=malloc(n2*sizeof(*tl2));
   error((tl1==NULL)||(tl2==NULL),1,"set_tls [check3.c]",
         "Unable to allocate tlist array");
   tl1[0]=NULL;

   for (i=1;i<=order;i++)
   {
      tl1[i]=tl2;
      nlp=nlp_max(th,i);

      for (j=0;j<=nlp;j++)
         tl2[j]=select_ltrees(th,i,j);

      tl2+=(nlp+1);
   }

   tls=tl1;
}


static void free_tls(void)
{
   free(tls[1]);
   free(tls);
}


static void alloc_dmw(theory_t th,int order)
{
   int i,j,l,nel;
   diagram_t *dm,**dma;
   dlist_t *dl;

   nvw=0;
   nlw=0;

   for (i=1;i<=order;i++)
   {
      nel=nel_max(th,i);

      for (j=2;j<=nel;j++)
      {
         dl=dls[i][j];

         if (dl!=NULL)
         {
            dma=(*dl).dma;

            for (l=0;l<(*dl).n;l++)
            {
               dm=dma[l];

               if (nvw<(*dm).nv)
                  nvw=(*dm).nv;
               if (nlw<(*dm).nl)
                  nlw=(*dm).nl;
            }
         }
      }
   }

   if (nvw>0)
   {
      dmw=alloc_dgm(nvw,nlw);
      rkw=malloc(nvw*sizeof(*rkw));
      error(rkw==NULL,1,"alloc_dmw [check3.c]","Unable to allocate rank array");
   }
   else
   {
      dmw=NULL;
      rkw=NULL;
   }

   if (nlw>0)
   {
      lnl=malloc(nlw*sizeof(*lnl));
      error(lnl==NULL,1,"alloc_dmw [check3.c]","Unable to allocate line list");
   }
   else
      lnl=NULL;
}


static void free_dmw(void)
{
   if (nvw>0)
   {
      free_dgm(dmw);
      free(rkw);
   }

   if (nlw>0)
      free(lnl);
}


static void alloc_aux(int order)
{
   int n1;

   n1=order+1;
   ki=malloc(5*n1*sizeof(*ki));
   rl=malloc(2*n1*sizeof(*rl));
   lta=malloc(n1*sizeof(*lta));
   ltd=malloc(n1*sizeof(*ltd));
   error((ki==NULL)||(rl==NULL)||(lta==NULL)||(ltd==NULL),1,
         "main [check3.c]","Unable to allocate auxiliary arrays");
   li=ki+n1;
   ji=li+n1;
   nl=ji+n1;
   nj=nl+n1;
   rj=rl+n1;
}


static void free_aux(void)
{
   free(ki);
   free(rl);
   free(lta);
   free(ltd);
}


static void set_nl(void)
{
   int i;

   ml=0;
   nl[0]=1;

   for (i=1;i<n;i++)
   {
      if (ki[i]==ki[i-1])
         nl[ml]+=1;
      else
      {
         ml+=1;
         nl[ml]=1;
      }
   }

   ml+=1;
}


static void set_rl(theory_t th)
{
   int i,j;

   j=0;

   for (i=0;i<ml;i++)
   {
      rl[i][0]=0;
      rl[i][1]=nlp_max(th,ki[j]);
      j+=nl[i];
   }
}


static void set_nj(void)
{
   int i;

   mj=0;
   nj[0]=1;

   for (i=1;i<n;i++)
   {
      if ((ki[i]==ki[i-1])&&(li[i]==li[i-1]))
         nj[mj]+=1;
      else
      {
         mj+=1;
         nj[mj]=1;
      }
   }

   mj+=1;
}


static int set_rj(theory_t th)
{
   int i,j;
   tlist_t *tl;

   j=0;

   for (i=0;i<mj;i++)
   {
      rj[i][0]=0;
      tl=tls[ki[j]][li[j]];

      if (tl!=NULL)
         rj[i][1]=(*tl).n-1;
      else
         return 0;

      j+=nj[i];
   }

   return 1;
}


static void set_lta(void)
{
   int i;
   tlist_t *tl;

   for (i=0;i<n;i++)
   {
      tl=tls[ki[i]][li[i]];
      lta[i]=(*tl).lta;
   }
}


static void set_ltd(void)
{
   int i;

   for (i=0;i<n;i++)
      ltd[i]=lta[i][ji[i]];
}


static void contract_ltd(theory_t th,int order,long *snp)
{
   int i,ne,nmx,nln;
   int con1,con2;
   long np,npm,nc;
   ltype_t lt;

   get_src_legs(th,n,ltd,&sl);
   get_vsets(n,ltd,&vs);

   if (th==PHI4)
      lt=SCALAR;
   else
      lt=GLUON;

   if (sl.lbl[0]==0)
   {
      if (n==1)
      {
         if (sl.ns[0]>2)
            nmx=(sl.ns[0]-1)/2;
         else
            nmx=0;
      }
      else
         nmx=sl.ns[0]/2;
   }
   else
      nmx=0;

   nln=0;

   for (i=0;i<sl.nt;i++)
   {
      if (sl.lbl[i]>0)
         nln+=sl.ns[i];
   }

   nln/=2;
   snp[0]=0L;
   snp[1]=0L;
   snp[2]=0L;

   for (i=0;i<=nmx;i++)
   {
      npm=init_contract(i,&sl);
      np=0L;

      while (np<npm)
      {
         np=next_contract(lnl,&nc);

         (*dmw).nv=nvw;
         (*dmw).nl=nlw;

         merge_ltrees(n,ltd,nln+i,lnl,dmw);

         ne=check_dgm(th,order,dmw);
         error(ne!=(n+sl.ns[0]-2*i),1,"contract_ltd [check3.c]",
               "Unexpected number of external lines");

         con1=ncparts(dmw);
         con2=lnlcon(&vs,nln+i,lnl);
         error(((con1==1)&&(con2!=1))||((con1>1)&&(con2==1)),1,
               "contract_ltd [check3.c]","Incorrect connectivity determination");

         if (con2==1)
         {
            snp[1]+=1L;

            if (has_tadpole(dmw,lt,&vls)==0)
               snp[2]+=1L;
         }
      }

      stop_contract();
      snp[0]+=npm;
   }
}


static void check_d2rmap(theory_t th,int order)
{
   int i,j,l,nel;
   int id[3],ir[3],idr[3];
   dlist_t *dl;
   tlist_t *tl;

   for (i=1;i<=order;i++)
   {
      id[0]=i;
      nel=nel_max(th,i);

      for (j=2;j<=nel;j++)
      {
         id[1]=j;
         dl=dls[i][j];

         if (dl!=NULL)
         {
            for (l=0;l<(*dl).n;l++)
            {
               id[2]=l;
               iD2R(th,id,ir);
               iR2D(th,ir,idr);

               error((idr[0]!=id[0])||(idr[1]!=id[1])||(idr[2]!=id[2]),1,
                     "check_d2rmap [check3.c]","Inconsistent D2R and R2D maps");

               tl=tls[ir[0]][ir[1]];

               (*dmw).nv=nvw;
               (*dmw).nl=nlw;
               ltd2dgm((*tl).lta[ir[2]],dmw);
               reorder_dgm(dmw,rkw);

               error(cmp_dgm(dmw,rkw,(*dl).dma[l],(*dl).rka[l])!=1,2,
                     "check_d2rmap [check3.c]","Inconsistent D2R and R2D maps");
            }
         }
      }
   }
}


int main(void)
{
   int order,is,na0,na1,na2;
   char name[5];
   long sm[3],snp[3];
   theory_t th;

   printf("\n");
   printf("Contraction of source fields\n");
   printf("----------------------------\n\n");

   init_src_legs(&sl);
   init_vsets(&vs);

   while (1)
   {
      printf("Specify theory, maximal order and root selection flag: ");
      (void)(scanf("%4s %d %d",name,&order,&is));

      th=theory(name);
      na0=trees(th,order);
      na1=ltrees(th,order);
      set_root(th,is);
      na2=cdgms(th,order);
      error(na2!=na1,1,"main [check3.c]",
            "Numbers of labeled tree and contracted diagrams do not match");

      printf("\n");
      printf("Total number of basic tree diagrams =   %d\n",na0);
      printf("Total number of labeled tree diagrams = %d\n",na1);

      vls=vertices(th,order);
      set_dls(th,order);
      set_tls(th,order);
      alloc_dmw(th,order);
      alloc_aux(order);

      for (k=1;k<=order;k++)
      {
         sm[0]=0L;
         sm[1]=0L;
         sm[2]=0L;

         for (n=1;n<=k;n++)
         {
            nmk=init_partition(n,k);
            nck=0L;

            while (nck<nmk)
            {
               nck=next_partition(ki);
               set_nl();
               set_rl(th);
               nml=init_mindex(ml,nl,rl,&cidl);
               ncl=0L;

               while (ncl<nml)
               {
                  ncl=next_mindex(cidl,li);
                  set_nj();

                  if (set_rj(th))
                  {
                     set_lta();
                     nmj=init_mindex(mj,nj,rj,&cidj);
                     ncj=0L;

                     while (ncj<nmj)
                     {
                        ncj=next_mindex(cidj,ji);
                        set_ltd();
                        contract_ltd(th,order,snp);

                        sm[0]+=snp[0];
                        sm[1]+=snp[1];
                        sm[2]+=snp[2];
                     }

                     stop_mindex(cidj);
                  }
               }

               stop_mindex(cidl);
            }

            stop_partition();
         }

         printf("Number of line lists at order %d: %ld\n",k,sm[0]);
         printf("               w/o disconnected: %ld\n",sm[1]);
         printf("                    w/o tapoles: %ld\n",sm[2]);
      }

      check_d2rmap(th,order);

      free_aux();
      free_dmw();
      free_tls();
      free_dls();

      printf("No errors found\n\n");
   }

   exit(0);
}
