
/*******************************************************************************
*
* File table3.c
*
* Copyright (C) 2016 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Table of the coefficients r,r' of the O(a) boundary corrections of the
* gradient-flow coupling (see modules/nompi/gbar/rfact.c and doc/bndimp.pdf).
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "utils.h"
#include "gbar.h"

static int ns,*Ts,*Ls,bcs,Ns;
static double c1s,cs;
static char type[NAME_SIZE];


static void read_parms(void)
{
   int i,ie;
   FILE *fin=NULL;

   fin=fopen("table3.in","r");
   error(fin==NULL,1,"read_parms [table3.c]",
         "Unable to open input parameter file");

   find_section(fin,"Lattice");
   ns=count_tokens(fin,"T");
   error((ns<1)||(ns!=count_tokens(fin,"L")),1,"read_parms [table3.c]",
         "Improper lists of T and L values");
   Ts=malloc(2*ns*sizeof(*Ts));
   Ls=Ts+ns;
   error(Ts==NULL,1,"read_parms [table3.c]",
         "Unable to allocate size arrays");
   read_iprms(fin,"T",ns,Ts);
   read_iprms(fin,"L",ns,Ls);
   read_line(fin,"bc","%d",&bcs);

   find_section(fin,"Theory");
   read_line(fin,"N","%d",&Ns);
   read_line(fin,"c1","%lf\n",&c1s);

   find_section(fin,"Coupling");
   read_line(fin,"type","%s\n",type);
   read_line(fin,"c","%lf\n",&cs);

   ie=0;

   for (i=0;i<ns;i++)
      ie|=((Ts[i]<4)||((Ts[i]%2)!=0)||(Ls[i]<4));

   error((ie!=0)||(bcs<1)||(bcs>2)||(Ns<=1)||(c1s>=0.125)||(cs<=0.0),1,
         "read_parms [table3.c]","Improper choice of input parameters");
   error((strcmp(type,"clover")!=0)&&(strcmp(type,"plaquette")!=0),1,
         "read_parms [table3.c]","Parameter type must be clover or plaquette");

   fclose(fin);
}


static void print_table(FILE *fdat,FILE *fdatp)
{
   int n;
   double sr,skr;
   double c0,t,k,ks,kt;
   double r[2],rs[2],rt[2];
   char ty0[2],ty1[3],ty2[3];

   ty0[0]=type[0];
   ty1[0]=type[0];
   ty2[0]=type[0];
   ty1[1]='s';
   ty2[1]='t';
   ty0[1]='\0';
   ty1[2]='\0';
   ty2[2]='\0';

   fprintf(fdat,"#\n");
   fprintf(fdat,"# Coefficients r_%s,r_%s,r_%s\n",ty0,ty1,ty2);
   fprintf(fdat,"#\n");

   if (bcs==1)
      fprintf(fdat,"# SF boundary conditions\n");
   else
      fprintf(fdat,"# Open-SF boundary conditions\n");

   fprintf(fdat,"# N = %d\n",Ns);
   c0=1.0-8.0*c1s;
   n=fdigits(c0);
   fprintf(fdat,"# c0 = %.*f, ",IMAX(n,1),c0);
   n=fdigits(c1s);
   fprintf(fdat,"c1 = %.*f\n",IMAX(n,1),c1s);
   n=fdigits(cs);
   fprintf(fdat,"# c = %.*f\n",IMAX(n,1),cs);
   fprintf(fdat,"#\n");

   fprintf(fdat,"#  T   L   r_%s*10^4  r_%s*10^4  r_%s*10^4",ty0,ty1,ty2);
   fprintf(fdat,"  4pi*k_%s*r_%s  4pi*k_%s*r_%s  4pi*k_%s*r_%s\n",
           ty0,ty0,ty1,ty1,ty2,ty2);

   if (bcs==2)
   {
      fprintf(fdatp,"#\n");
      fprintf(fdatp,"# Coefficients r'_%s,r'_%s,r'_%s\n",ty0,ty1,ty2);
      fprintf(fdatp,"#\n");

      if (bcs==1)
         fprintf(fdatp,"# SF boundary conditions\n");
      else
         fprintf(fdatp,"# Open-SF boundary conditions\n");

      fprintf(fdatp,"# N = %d\n",Ns);
      c0=1.0-8.0*c1s;
      n=fdigits(c0);
      fprintf(fdatp,"# c0 = %.*f, ",IMAX(n,1),c0);
      n=fdigits(c1s);
      fprintf(fdatp,"c1 = %.*f\n",IMAX(n,1),c1s);
      n=fdigits(cs);
      fprintf(fdatp,"# c = %.*f\n",IMAX(n,1),cs);
      fprintf(fdatp,"#\n");

      fprintf(fdatp,"#  T   L   r'_%s*10^4  r'_%s*10^4  r'_%s*10^4",
              ty0,ty1,ty2);
      fprintf(fdatp,"  4pi*k_%s*r'_%s  4pi*k_%s*r'_%s  4pi*k_%s*r'_%s\n",
              ty0,ty0,ty1,ty1,ty2,ty2);
   }

   sr=1.0e+04;
   skr=16.0*atan(1.0);

   for (n=0;n<ns;n++)
   {
      t=cs*(double)(Ls[n]);
      t=(t*t)/8.0;
      k=kfact(ty0,Ns,Ts[n],Ls[n],bcs,c1s,Ts[n]/2,t);
      ks=kfact(ty1,Ns,Ts[n],Ls[n],bcs,c1s,Ts[n]/2,t);
      kt=kfact(ty2,Ns,Ts[n],Ls[n],bcs,c1s,Ts[n]/2,t);

      rfact(ty0,Ns,Ts[n],Ls[n],bcs,c1s,t,r);
      rfact(ty1,Ns,Ts[n],Ls[n],bcs,c1s,t,rs);
      rfact(ty2,Ns,Ts[n],Ls[n],bcs,c1s,t,rt);

      fprintf(fdat," %3d %3d % 9.4f % 10.4f % 10.4f ",
              Ts[n],Ls[n],sr*r[0],sr*rs[0],sr*rt[0]);
      fprintf(fdat,"% 12.4f % 12.4f % 13.4f\n",
              skr*k*r[0],skr*ks*rs[0],skr*kt*rt[0]);

      if (bcs==2)
      {
         fprintf(fdatp," %3d %3d % 10.4f % 11.4f % 11.4f ",
                 Ts[n],Ls[n],sr*r[1],sr*rs[1],sr*rt[1]);
         fprintf(fdatp,"% 13.4f % 13.4f % 14.4f\n",
                 skr*k*r[1],skr*ks*rs[1],skr*kt*rt[1]);
      }
   }
}


int main(int argc,char *argv[])
{
   FILE *fdat,*fdatp;

   read_parms();

   fdat=fopen("plots/table3.dat","w");
   error(fdat==NULL,1,"main [table3.c]","Unable to open output file");

   if (bcs==2)
   {
      fdatp=fopen("plots/table3p.dat","w");
      error(fdat==NULL,1,"main [table3.c]","Unable to open output file");
   }
   else
      fdatp=NULL;

   print_table(fdat,fdatp);
   fclose(fdat);
   if (bcs==2)
      fclose(fdatp);

   printf("\n");
   printf("O(a) boundary corrections of the gradient-flow coupling ");
   printf("(%s definition)\n\n",type);

   if (bcs==1)
      printf("Table printed to plots/table3.dat\n\n");
   else
      printf("Tables printed to plots/table3.dat and plots/table3p.dat\n\n");

   exit(0);
}
