
/*******************************************************************************
*
* File check7.c
*
* Copyright (C) 2015, 2017 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the programs cm3x3_ren2sl3_poly() and cm3x3_ren2su3_poly().
*
*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "su3.h"
#include "utils.h"
#include "random.h"
#include "su3fcts.h"

#define NTEST 100
#define SEED 376

static sl3_alg_dble wa[8] ALIGNED16;
static su3_dble wu[8] ALIGNED16;
static su3_dble wx[8] ALIGNED16;
static su3_dble X ALIGNED16;
static su3_dble Y ALIGNED16;
static su3_dble one ALIGNED16;
static su3_dble zero ALIGNED16;


static double dev_uv(su3_dble *u,su3_dble *v)
{
   int i;
   double r[18],dev,dmax;

   r[ 0]=(*u).c11.re-(*v).c11.re;
   r[ 1]=(*u).c11.im-(*v).c11.im;
   r[ 2]=(*u).c12.re-(*v).c12.re;
   r[ 3]=(*u).c12.im-(*v).c12.im;
   r[ 4]=(*u).c13.re-(*v).c13.re;
   r[ 5]=(*u).c13.im-(*v).c13.im;

   r[ 6]=(*u).c21.re-(*v).c21.re;
   r[ 7]=(*u).c21.im-(*v).c21.im;
   r[ 8]=(*u).c22.re-(*v).c22.re;
   r[ 9]=(*u).c22.im-(*v).c22.im;
   r[10]=(*u).c23.re-(*v).c23.re;
   r[11]=(*u).c23.im-(*v).c23.im;

   r[12]=(*u).c31.re-(*v).c31.re;
   r[13]=(*u).c31.im-(*v).c31.im;
   r[14]=(*u).c32.re-(*v).c32.re;
   r[15]=(*u).c32.im-(*v).c32.im;
   r[16]=(*u).c33.re-(*v).c33.re;
   r[17]=(*u).c33.im-(*v).c33.im;

   dmax=0.0;

   for (i=0;i<18;i++)
   {
      dev=fabs(r[i]);
      if (dev>dmax)
         dmax=dev;
   }

   return dmax;
}


static void sl2su(int n,sl3_alg_dble *a)
{
   int i;

   for (i=0;i<n;i++)
   {
      a[i].c1.re=0.0;
      a[i].c2.re=0.0;

      a[i].c4.re=-a[i].c3.re;
      a[i].c4.im=a[i].c3.im;

      a[i].c6.re=-a[i].c5.re;
      a[i].c6.im=a[i].c5.im;

      a[i].c8.re=-a[i].c7.re;
      a[i].c8.im=a[i].c7.im;
   }
}


static void tilt_uarray(double eps,int n,su3_dble *u)
{
   int i;
   complex_dble z;

   for (i=0;i<n;i++)
   {
      gauss_dble((double*)(&z),2);
      u[i].c11.re+=eps*z.re;
      u[i].c11.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c12.re+=eps*z.re;
      u[i].c12.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c13.re+=eps*z.re;
      u[i].c13.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c21.re+=eps*z.re;
      u[i].c21.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c22.re+=eps*z.re;
      u[i].c22.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c23.re+=eps*z.re;
      u[i].c23.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c31.re+=eps*z.re;
      u[i].c31.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c32.re+=eps*z.re;
      u[i].c32.im+=eps*z.im;

      gauss_dble((double*)(&z),2);
      u[i].c33.re+=eps*z.re;
      u[i].c33.im+=eps*z.im;
   }
}


static double chk_unitarity(int n,su3_dble *u)
{
   int i,j;
   double d,dmax;

   dmax=0.0;

   for (i=0;i<n;i++)
   {
      cm3x3_dagger(u+i,&X);
      cm3x3_add(u+i,&X);

      for (j=0;j<i;j++)
      {
         su3xsu3dag(u+j,u+i-1-j,&Y);
         cm3x3_add(&Y,&X);
      }

      d=dev_uv(&X,&zero);

      if (d>dmax)
         dmax=d;
   }

   return dmax;
}


static double chk_unimod(int n,su3_dble *u)
{
   int i;
   complex_dble z;
   double d,dmax;

   cm3x3_log_poly(n,u,wx);
   dmax=0.0;

   for (i=0;i<n;i++)
   {
      z.re=wx[i].c11.re+wx[i].c22.re+wx[i].c33.re;
      z.im=wx[i].c11.im+wx[i].c22.im+wx[i].c33.im;
      d=sqrt(z.re*z.re+z.im*z.im);

      if (d>dmax)
         dmax=d;
   }

   return dmax;
}


int main(void)
{
   int n,i,j;
   double r,d,dmax[9];

   printf("\n");
   printf("Check of the programs cm3x3_ren2sl3_poly() "
          "and cm3x3_ren2su3_poly()\n");
   printf("-------------------------------------------"
          "------------------------\n\n");

#if (defined AVX)
   printf("Using AVX instructions\n");
#elif (defined x64)
   printf("Using SSE3 instructions and up to 16 xmm registers\n");
#endif

   printf("Test performed on %d random matrices\n\n",NTEST);

   rlxd_init(1,SEED);
   cm3x3_unity(1,&one);
   cm3x3_zero(1,&zero);
   for (i=0;i<9;i++)
      dmax[i]=0.0;

   for (i=0;i<NTEST;i++)
   {
      for (n=1;n<=8;n++)
      {
         gauss_dble((double*)(wa),n*16);
         r=0.1;

         for (j=0;j<n;j++)
            sl3_mulr(&r,wa+j,wa+j);

         pexp_sl3_alg(n,wa,wu);
         d=chk_unimod(n,wu);
         if (d>dmax[0])
            dmax[0]=d;

         tilt_uarray(1.0e-12,n,wu);
         d=chk_unimod(n,wu);
         dmax[1]+=d;

         cm3x3_ren2sl3_poly(n,wu);
         d=chk_unimod(n,wu);
         if (d>dmax[2])
            dmax[2]=d;

         gauss_dble((double*)(wa),n*16);
         r=0.1;

         for (j=0;j<n;j++)
            sl3_mulr(&r,wa+j,wa+j);

         sl2su(n,wa);
         pexp_sl3_alg(n,wa,wu);
         d=chk_unitarity(n,wu);
         if (d>dmax[3])
            dmax[3]=d;
         d=chk_unimod(n,wu);
         if (d>dmax[4])
            dmax[4]=d;

         tilt_uarray(1.0e-12,n,wu);
         d=chk_unitarity(n,wu);
         dmax[5]+=d;
         d=chk_unimod(n,wu);
         dmax[6]+=d;

         cm3x3_ren2su3_poly(n,wu);
         d=chk_unitarity(n,wu);
         if (d>dmax[7])
            dmax[7]=d;
         d=chk_unimod(n,wu);
         if (d>dmax[8])
            dmax[8]=d;
      }
   }

   printf("Check of cm3x3_ren2sl3_poly():\n");
   printf("Determinant deviation (initial)               = %.1e\n",
          dmax[0]);
   printf("Determinant deviation (after tilt)            = %.1e\n",
          dmax[1]/(double)(8*NTEST));
   printf("Determinant deviation (after renormalization) = %.1e\n\n",
          dmax[2]);

   printf("Check of cm3x3_ren2su3_poly():\n");
   printf("Unitarity deviation (initial)                 = %.1e\n",
          dmax[3]);
   printf("Determinant deviation (initial)               = %.1e\n",
          dmax[4]);
   printf("Unitarity deviation (after tilt)              = %.1e\n",
          dmax[5]/(double)(8*NTEST));
   printf("Determinant deviation (after tilt)            = %.1e\n",
          dmax[6]/(double)(8*NTEST));
   printf("Unitarity deviation (after renormalization)   = %.1e\n",
          dmax[7]);
   printf("Determinant deviation (after renormalization) = %.1e\n\n",
          dmax[8]);

   exit(0);
}
