
/*******************************************************************************
*
* File check5.c
*
* Copyright (C) 2015 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the program rtrees().
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "utils.h"
#include "tupel.h"
#include "contract.h"
#include "trees.h"

static char *vtname[VTYPES+1]={"SOURCE","EXTERNAL",
                               "VS","VG","VGM","VFP","VQG",
                               "VSM","VGF","VQM",
                               "VTYPES"};


static theory_t theory(char *name)
{
   if (strcmp("PHI4",name)==0)
      return PHI4;
   else if (strcmp("YM",name)==0)
      return YM;
   else if (strcmp("QCD",name)==0)
      return QCD;
   else if (strcmp("LYM",name)==0)
      return LYM;
   else if (strcmp("LQCD",name)==0)
      return LQCD;
   else
      error(1,1,"theory [check5.c]","Unknown theory");

   return THEORIES;
}


static void print_tp(tupel_t *tp)
{
   int n,*i,*im;

   n=(*tp).n;

   if (n>0)
   {
      i=(*tp).i;
      im=i+n;

      printf("[%d",i[0]);
      i+=1;

      for (;i<im;i++)
         printf(",%d",i[0]);

      printf("]");
   }
}


static int rt_order(rtree_t *rt)
{
   int n;
   rtree_t *br,*brm;

   n=(*rt).v[0];
   br=(*rt).br;
   brm=br+(*rt).nbr;

   for (;br<brm;br++)
      n+=rt_order(br);

   return n;
}


static int rt_nVX(rtree_t *rt,vtype_t vt,vlist_t *vls)
{
   int n,*v;
   vertex_t *vx;
   rtree_t *br,*brm;

   v=(*rt).v;
   vx=(*vls).v[v[0]]+v[1];

   if (vt==(*vx).type)
      n=1;
   else
      n=0;

   br=(*rt).br;
   brm=br+(*rt).nbr;

   for (;br<brm;br++)
      n+=rt_nVX(br,vt,vls);

   return n;
}


int main(void)
{
   int order,is;
   int na0,na1,ncl,icl,iv;
   int n,k;
   char name[5];
   theory_t th;
   vlist_t vls;
   tupel_t **tpa;
   rcls_t *rc;
   rtree_t **rta;
   cnum_t **cna;
   rlist_t *rl;

   printf("\n");
   printf("Check of the program rtrees()\n");
   printf("-----------------------------\n\n");

   while (1)
   {
      printf("Specify theory, maximal order and root selection flag: ");
      (void)(scanf("%4s %d %d",name,&order,&is));

      th=theory(name);
      vls=vertices(th,order);
      na0=tmap(th,order,is);
      na1=rtrees(th,order);

      printf("\n");
      printf("Total number of labeled tree diagrams = %d\n",na0);
      printf("Number of diagrams kept by rtrees()   = %d\n\n",na1);

      rc=rtree_classes(th);
      ncl=rc[0].cls[0].n;
      tpa=rc[0].cls[0].tpa;

      printf("There are %d classes of diagrams\n\n",ncl);

      printf("Counterterm vertices = ");

      for (iv=0;iv<rc[0].nvt;iv++)
      {
         if (iv>0)
            printf(", %s",vtname[(int)(rc[0].vt[iv])]);
         else
            printf("%s",vtname[(int)(rc[0].vt[iv])]);
      }

      printf("\n\n");

      printf("Numbers of tree diagrams in a given class:\n");
      printf("  class        no of diagrams\n");

      for (icl=0;icl<ncl;icl++)
      {
         rl=select_rtrees(th,icl);
         n=(*rl).n;
         rta=(*rl).rta;
         cna=(*rl).cna;

         printf("  ");
         print_tp(tpa[icl]);
         printf("      % 7d\n",n);

         for (k=0;k<n;k++)
         {
            error(is_zero_cnum(cna[k]),1,"main [check5.c]",
                  "Diagram has vanishing coefficient");

            error(rt_order(rta[k])!=tpa[icl][0].i[0],1,"main [check5.c]",
                  "Unexpected order of the diagrams");

            for (iv=0;iv<rc[0].nvt;iv++)
            {
               error(rt_nVX(rta[k],rc[0].vt[iv],&vls)!=tpa[icl][0].i[iv+1],1,
                     "main [check5.c]","Incorrect no of counterterm vertices");
            }
         }
      }

      printf("\n");
   }

   exit(0);
}
