
/*******************************************************************************
*
* File check6.c
*
* Copyright (C) 2015, 2017 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Check of the program Agf().
*
*******************************************************************************/

#define MAIN_PROGRAM

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include "mpi.h"
#include "sl3.h"
#include "random.h"
#include "pmdb.h"
#include "utils.h"
#include "lattice.h"
#include "su3fcts.h"
#include "linalg.h"
#include "aflds.h"
#include "vxfcts.h"
#include "global.h"

static su3_dble vd ALIGNED16;
static su3_dble X ALIGNED16;
static su3_dble Y ALIGNED16;


static void new_fld(int n,sl3_alg_dble **ad)
{
   int i;

   for (i=0;i<n;i++)
   {
      if (cpr[0]==(NPROC0-1))
         gauss_dble((double*)(ad[i]),16*(4*VOLUME+7*(BNDRY/4)+3));
      else
         gauss_dble((double*)(ad[i]),16*(4*VOLUME+7*(BNDRY/4)));

      set_adbc(ad[i]);
   }
}


static void global_transform(int n,sl3_alg_dble **ad)
{
   int i,ix,ifc;
   sl3_alg_dble *fld;

   for (i=0;i<n;i++)
   {
      fld=ad[i];

      for (ix=(VOLUME/2);ix<VOLUME;ix++)
      {
         for (ifc=0;ifc<8;ifc++)
         {
            sl3_to_su3(fld,&X);
            su3xsu3(&vd,&X,&Y);
            su3xsu3dag(&Y,&vd,&X);
            su3_to_sl3(&X,fld);
            fld+=1;
         }
      }
   }
}


static void random_vec(int *svec)
{
   int mu,bs[4];
   double r[4];

   bs[0]=NPROC0*L0;
   bs[1]=NPROC1*L1;
   bs[2]=NPROC2*L2;
   bs[3]=NPROC3*L3;
   svec[0]=0;
   svec[1]=0;
   svec[2]=0;
   svec[3]=0;

   while ((abs(svec[1])+abs(svec[2])+abs(svec[3]))==0)
   {
      ranlxd(r,4);

      for (mu=1;mu<4;mu++)
      {
         svec[mu]=(int)((double)(bs[mu])*r[mu]);
         if (svec[mu]>(bs[mu]/2))
            svec[mu]-=bs[mu];
      }
   }

   MPI_Bcast(svec,4,MPI_INT,0,MPI_COMM_WORLD);
}


static void shift_flds(int n,int *s,sl3_alg_dble **ad)
{
   int i;

   for (i=0;i<n;i++)
      shift_ad(s,ad[i]);
}


int main(int argc,char *argv[])
{
   int my_rank,bc,i,s[4];
   double d;
   complex_dble z,w;
   sl3_alg_dble **wad;
   FILE *flog=NULL;

   MPI_Init(&argc,&argv);
   MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

   if (my_rank==0)
   {
      flog=freopen("check6.log","w",stdout);

      printf("\n");
      printf("Check of the program Agf()\n");
      printf("--------------------------\n\n");

      printf("%dx%dx%dx%d lattice, ",NPROC0*L0,NPROC1*L1,NPROC2*L2,NPROC3*L3);
      printf("%dx%dx%dx%d process grid, ",NPROC0,NPROC1,NPROC2,NPROC3);
      printf("%dx%dx%dx%d local lattice\n\n",L0,L1,L2,L3);

      bc=find_opt(argc,argv,"-bc");

      if (bc!=0)
         error_root(sscanf(argv[bc+1],"%d",&bc)!=1,1,"main [check6.c]",
                    "Syntax: check6 [-bc <type>]");
      fflush(flog);
   }

   MPI_Bcast(&bc,1,MPI_INT,0,MPI_COMM_WORLD);
   set_th_parms(LYM,bc,0);
   print_th_parms();
   start_ranlux(0,12345);
   geometry();
   alloc_wad(4);
   alloc_wcd(2);
   wad=reserve_wad(4);

   if (my_rank==0)
      printf("Global gauge invariance:\n");

   random_su3_dble(&vd);
   MPI_Bcast(&vd,18,MPI_DOUBLE,0,MPI_COMM_WORLD);
   new_fld(3,wad);
   Agf(wad[0],wad[1]);
   global_transform(2,wad);
   Agf(wad[0],wad[2]);
   mulr_add_calg(4*VOLUME,wad[2],wad[1],-1.0);
   d=norm_square_calg(4*VOLUME,1,wad[2])/
      norm_square_calg(4*VOLUME,1,wad[1]);

   if (my_rank==0)
   {
      printf("deviation = %.1e\n\n",sqrt(d));
      printf("Translation invariance:\n");
   }

   for (i=0;i<4;i++)
   {
      new_fld(3,wad);
      Agf(wad[0],wad[1]);
      random_vec(s);
      shift_flds(2,s,wad);
      Agf(wad[0],wad[2]);
      mulr_add_calg(4*VOLUME,wad[2],wad[1],-1.0);
      d=norm_square_calg(4*VOLUME,1,wad[2])/
         norm_square_calg(4*VOLUME,1,wad[1]);

      if (my_rank==0)
      {
         printf("s= % 3d,% 3d,% 3d,% 3d, deviation = %.1e\n",
                s[0],s[1],s[2],s[3],sqrt(d));
         fflush(flog);
      }
   }

   if (my_rank==0)
   {
      printf("\n");
      printf("Linearity:\n");
      fflush(flog);
   }

   new_fld(4,wad);
   Agf(wad[0],wad[2]);
   Agf(wad[1],wad[3]);
   z.re=0.7896;
   z.im=-1.543;
   mulc_add_calg(4*VOLUME,wad[2],wad[3],z);
   mulc_add_calg(4*VOLUME,wad[0],wad[1],z);
   Agf(wad[0],wad[3]);

   mulr_add_calg(4*VOLUME,wad[2],wad[3],-1.0);
   d=norm_square_calg(4*VOLUME,1,wad[2])/
      norm_square_calg(4*VOLUME,1,wad[3]);

   if (my_rank==0)
   {
      printf("deviation = %.1e\n\n",sqrt(d));
      printf("Comparison with action:\n");
      fflush(flog);
   }

   new_fld(1,wad);
   assign_calgdag2calg(4*VOLUME,wad[0],wad[1]);

   Agf(wad[0],wad[2]);
   z=scalar_prod_calg(4*VOLUME,1,wad[1],wad[2]);
   z.re*=0.5;
   z.im*=0.5;

   w=gfix_action(wad[0],wad[0]);
   w.re-=z.re;
   w.im-=z.im;
   d=(w.re*w.re+w.im*w.im)/(z.re*z.re+z.im*z.im);

   if (my_rank==0)
   {
      printf("deviation = %.1e\n\n",sqrt(d));
      fclose(flog);
   }

   MPI_Finalize();
   exit(0);
}
