
/*******************************************************************************
*
* File sl3.h
*
* Copyright (C) 2015 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Type definitions and macros for sl(3,C) matrices.
*
*******************************************************************************/

#ifndef SL3_H
#define SL3_H

#ifndef SU3_H
#include "su3.h"
#endif

typedef struct
{
   complex c1,c2,c3,c4,c5,c6,c7,c8;
} sl3_alg;

typedef struct
{
   complex_dble c1,c2,c3,c4,c5,c6,c7,c8;
} sl3_alg_dble;


/*******************************************************************************
*
* Macros for variables of type sl3_alg
*
*******************************************************************************/

/*
* r+=s
*/

#define _sl3_alg_add_assign(r,s) \
   (r).c1.re+=(s).c1.re; \
   (r).c1.im+=(s).c1.im; \
   (r).c2.re+=(s).c2.re; \
   (r).c2.im+=(s).c2.im; \
   (r).c3.re+=(s).c3.re; \
   (r).c3.im+=(s).c3.im; \
   (r).c4.re+=(s).c4.re; \
   (r).c4.im+=(s).c4.im; \
   (r).c5.re+=(s).c5.re; \
   (r).c5.im+=(s).c5.im; \
   (r).c6.re+=(s).c6.re; \
   (r).c6.im+=(s).c6.im; \
   (r).c7.re+=(s).c7.re; \
   (r).c7.im+=(s).c7.im; \
   (r).c8.re+=(s).c8.re; \
   (r).c8.im+=(s).c8.im;


/*
* r-=s
*/

#define _sl3_alg_sub_assign(r,s) \
   (r).c1.re-=(s).c1.re; \
   (r).c1.im-=(s).c1.im; \
   (r).c2.re-=(s).c2.re; \
   (r).c2.im-=(s).c2.im; \
   (r).c3.re-=(s).c3.re; \
   (r).c3.im-=(s).c3.im; \
   (r).c4.re-=(s).c4.re; \
   (r).c4.im-=(s).c4.im; \
   (r).c5.re-=(s).c5.re; \
   (r).c5.im-=(s).c5.im; \
   (r).c6.re-=(s).c6.re; \
   (r).c6.im-=(s).c6.im; \
   (r).c7.re-=(s).c7.re; \
   (r).c7.im-=(s).c7.im; \
   (r).c8.re-=(s).c8.re; \
   (r).c8.im-=(s).c8.im;


/*
* r=c*s, c real
*/

#define _sl3_alg_mulr_assign(r,c,s) \
   (r).c1.re=(c)*(s).c1.re; \
   (r).c1.im=(c)*(s).c1.im; \
   (r).c2.re=(c)*(s).c2.re; \
   (r).c2.im=(c)*(s).c2.im; \
   (r).c3.re=(c)*(s).c3.re; \
   (r).c3.im=(c)*(s).c3.im; \
   (r).c4.re=(c)*(s).c4.re; \
   (r).c4.im=(c)*(s).c4.im; \
   (r).c5.re=(c)*(s).c5.re; \
   (r).c5.im=(c)*(s).c5.im; \
   (r).c6.re=(c)*(s).c6.re; \
   (r).c6.im=(c)*(s).c6.im; \
   (r).c7.re=(c)*(s).c7.re; \
   (r).c7.im=(c)*(s).c7.im; \
   (r).c8.re=(c)*(s).c8.re; \
   (r).c8.im=(c)*(s).c8.im;

/*
* r+=c*s, c real
*/

#define _sl3_alg_mulr_add_assign(r,c,s) \
   (r).c1.re+=(c)*(s).c1.re; \
   (r).c1.im+=(c)*(s).c1.im; \
   (r).c2.re+=(c)*(s).c2.re; \
   (r).c2.im+=(c)*(s).c2.im; \
   (r).c3.re+=(c)*(s).c3.re; \
   (r).c3.im+=(c)*(s).c3.im; \
   (r).c4.re+=(c)*(s).c4.re; \
   (r).c4.im+=(c)*(s).c4.im; \
   (r).c5.re+=(c)*(s).c5.re; \
   (r).c5.im+=(c)*(s).c5.im; \
   (r).c6.re+=(c)*(s).c6.re; \
   (r).c6.im+=(c)*(s).c6.im; \
   (r).c7.re+=(c)*(s).c7.re; \
   (r).c7.im+=(c)*(s).c7.im; \
   (r).c8.re+=(c)*(s).c8.re; \
   (r).c8.im+=(c)*(s).c8.im;

#endif
