
/*******************************************************************************
*
* File pbranch.c
*
* Copyright (C) 2014 Martin Luescher
*
* This software is distributed under the terms of the GNU General Public
* License (GPL)
*
* Drawing of branch diagrams.
*
* The externally accessible functions are
*
*   void draw_branch(branch_t *br,vlist *vl)
*     Writes plot commands representing the branch "br" to the currently
*     open plot file. The properties of the vertices in the branch are
*     retrieved from the vertex list "vl" (see diagrams/vertices.c).
*
* Notes:
*
* This program is to be used together with the plot tools provided by the
* module ptools.c. Assuming a plot file has been opened using start_plot(),
* a branch diagram "br" can be added to the plot by the commands
*
*  double pos1[2],pos2[2];
*
*  start_bbox();
*  draw_branch(br,&vl);
*  end_bbox(pos1,pos2);
*
*  start_diagram(pos1,pos2);
*  draw_branch(br,&vl);
*  end_diagram();
*
* where the vlist_t "vl" is assumed to have been set somewhere before these
* lines.
*
*******************************************************************************/

#define PBRANCH_C

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "utils.h"
#include "rvt.h"
#include "plot.h"

#define LINE 14.0
#define VSEP 32.0

static double phi,dphi;


static void set_phi(int ie,int n)
{
   if (ie==0)
   {
      dphi=360.0/(double)(n);
      phi=0.0;
   }
   else if (ie==1)
   {
      dphi=360.0/(double)(n+1);
      phi=dphi;
   }
   else if (ie==2)
   {
      dphi=360.0/(double)(n+1);
      phi=180.0+dphi;
   }
   else if (ie==3)
   {
      if (n<=2)
      {
         dphi=180.0/(double)(n+1);
         phi=dphi;
      }
      else
      {
         dphi=60.0/(double)(n-1);
         phi=60.0;
      }
   }
   else
   {
      if (n<=2)
      {
         dphi=180.0/(double)(n+1);
         phi=180.0+dphi;
      }
      else
      {
         dphi=60.0/(double)(n-1);
         phi=240.0;
      }
   }
}


static void draw_brv(ltype_t lo,ltype_t ln,vertex_t *v)
{
   int n,nl,i,j,ie,iw;
   double vec[2];

   vec[0]=VSEP;
   vec[1]=0.0;

   draw_line(ln,0.0,vec);
   n=0;

   for (i=0;i<(int)(LTYPES);i++)
      n+=(*v).nl[i];

   n-=(lo!=LTYPES);
   n-=(ln!=LTYPES);

   if ((lo==LTYPES)&&(ln==LTYPES))
      ie=0;
   else if ((lo==LTYPES)&&(ln!=LTYPES))
      ie=1;
   else if ((lo!=LTYPES)&&(ln==LTYPES))
      ie=2;
   else
      ie=3;

   if (ie==3)
      set_phi(ie,(n+1)/2);
   else
      set_phi(ie,n);

   vec[0]=LINE;
   iw=0;

   for (i=0;i<(int)(LTYPES);i++)
   {
      nl=(*v).nl[i]-(i==(int)(rvt[lo]))-(i==(int)(ln));

      for (j=0;j<nl;j++)
      {
         draw_line((ltype_t)(i),phi,vec);
         phi+=dphi;
         iw+=1;

         if ((ie==3)&&(iw==((n+1)/2)))
         {
            ie=4;
            set_phi(ie,n/2);
         }
      }
   }

   draw_vertex((*v).type);
   vec[0]=VSEP;
   move_point(0.0,vec);
}


void draw_branch(branch_t *br,vlist_t *vl)
{
   int nv,i;
   ltype_t lo,ln;
   vertex_t *v;

   nv=(*br).nv;

   for (i=0;i<nv;i++)
   {
      if (i==0)
         lo=LTYPES;
      else
         lo=(*br).l[i-1];

      if (i==(nv-1))
         ln=LTYPES;
      else
         ln=(*br).l[i];

      v=(*vl).v[(*br).v[i][0]]+(*br).v[i][1];
      draw_brv(lo,ln,v);
   }
}
